//-----------------------------------------------------------------------------
/// \file	Main.cpp
/// \date	12-Jan-2005
/// \author	Rob Bateman
/// \brief	This example creates a single global callback function that acts as
/// 		an 'idle' function for the entire plugin. Use this when you want some
///			global update mechanism within your plugin. Ideally, it is sensible to
/// 		use the Callbacks as sparingly as possible, ie whilst idle might seem
///
///
///			The MEventMessage class allows you to create these callback functions,
///			which essentially behave as a script job. Infact, to get the list of
/// 		valid callback types, you can run the following mel command
///
/// 				 print `scriptJob -listEvents`;
///
///			That should give you the following list
///
///				angularUnitChanged
///				timeUnitChanged
///				linearUnitChanged
///				timeChanged
///				Undo
///				Redo
///				DisplayRGBColorChanged
///				ToolChanged
///				RecentCommandChanged
///				idleHigh
///				idle
///				quitApplication
///				DisplayColorChanged
///				displayLayerChange
///				displayLayerManagerChange
///				displayLayerAdded
///				displayLayerDeleted
///				displayLayerVisibilityChanged
///				renderLayerChange
///				renderLayerManagerChange
///				SelectionChanged
///				SelectModeChanged
///				SelectTypeChanged
///				SelectPreferenceChanged
///				DagObjectCreated
///				lightLinkingChanged
///				lightLinkingChangedNonSG
///				workspaceChanged
///				NewSceneOpened
///				SceneOpened
///				SceneImported
///				PreFileNewOrOpened
///				constructionHistoryChanged
///				nurbsCurveRebuildPrefsChanged
///				nurbsToPolygonsPrefsChanged
///				linearToleranceChanged
///				angularToleranceChanged
///				SetModified
///				NameChanged
///				deleteAll
///				ColorIndexChanged
///				nurbsToSubdivPrefsChanged
///				snapModeChanged
///				SelectPriorityChanged
///				axisAtOriginChanged
///				gridDisplayChanged
///				setEditorChanged
///				MenuModeChanged
///				playbackRangeChanged
///				playbackRangeSliderChanged
///				currentSoundNodeChanged
///				glFrameTrigger
///				RebuildUIValues
///
//-----------------------------------------------------------------------------

// required to link the libraries
#ifdef WIN32
	#define NT_PLUGIN
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
#endif

// under WIN32 we have to 'export' these functions so that they are visible
// inside the dll. If maya can't see the functions, it can't load the plugin!
#ifdef WIN32
	#define EXPORT __declspec(dllexport)
#else
	#define EXPORT
#endif

#include <maya/MFnPlugin.h>
#include <maya/MEventMessage.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>
#include <maya/MPoint.h>
#include <maya/M3dView.h>
#include <time.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

/// Having created a callback function within maya, we need to store the generated
/// MCallbackId so that we can delete the callback.
///
MCallbackId g_Callback;

int count=0;


/// The function that performs our idle processing
///
void IdleFunc( void* /* ignore */ ) {

	/// just print some information out
	MGlobal::displayInfo("IdleFunc()");

	++count;

	M3dView view = M3dView::active3dView();

	int w = view.portWidth();
	int h = view.portHeight();

	if(count>h)
		count = 0;

	view.beginOverlayDrawing ();

		view.clearOverlayPlane ();

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0,w,h,0,-100,100);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		glBegin(GL_LINES);
			glVertex2i(0,count);
			glVertex2i(w,count);
		glEnd();

	view.endOverlayDrawing ();
}




/// \brief	This function is called once when our plugin is loaded. We can use
///			it to register any custom nodes, mel functions etc.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus initializePlugin( MObject obj )
{
	// we need to use the plugin function set to register a new mel command with maya.
	// we can provide some info about the author of the plugin, the version number, and we
	// can also specify a required version of maya, though usually "Any" will do...
	//
	MFnPlugin fnPlugin( obj, "Rob Bateman", "1.0", "Any");

	/// When creating the callback for the idle function, we need to specify a callback
	/// type. In this case we are using "idle", however it can be anyone of the strings
	/// that `scriptJob -listEvents` returns;
	/// The second parameter is a pointer to the callback function. The third is a user
	/// data parameter (which is explained in MEventMessage2 example). In this case
	/// we can use NULL since we dont want any user data
	///
	g_Callback = MEventMessage::addEventCallback(
					"idle", IdleFunc, NULL );

	return MS::kSuccess;
}

/// \brief	This function is called once when our plugin is unloaded. We need
/// 		to tell maya which mel funcs, nodes etc we are removing.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus uninitializePlugin( MObject obj)
{
	// use the plugin function set to unload the plugin
	MFnPlugin fnPlugin( obj );

	/// make sure we delete the callback before the plugin quits !!!
	/// Maya is likely to crash otherwise!
	MEventMessage::removeCallback( g_Callback );

	return MS::kSuccess;
}


